@@ -1,20 +1,13 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-###
-### Author:  Chris Iatrou (ichrispa@core-vector.net)
-### Version: rev 13
-###
-### This program was created for educational purposes and has been
-### contributed to the open62541 project by the author. All licensing
-### terms for this source is inherited by the terms and conditions
-### specified for by the open62541 project (see the projects readme
-### file for more information on the LGPL terms and restrictions).
-###
-### This program is not meant to be used in a production environment. The
-### author is not liable for any complications arising due to the use of
-### this program.
-###
+### This Source Code Form is subject to the terms of the Mozilla Public
+### License, v. 2.0. If a copy of the MPL was not distributed with this
+### file, You can obtain one at http://mozilla.org/MPL/2.0/.
+
+###    Copyright 2014-2015 (c) TU-Dresden (Author: Chris Iatrou)
+###    Copyright 2014-2017 (c) Fraunhofer IOSB (Author: Julius Pfrommer)
+###    Copyright 2016-2017 (c) Stefan Profanter, fortiss GmbH
 
 from __future__ import print_function
 import sys
@@ -22,10 +15,15 @@ import xml.dom.minidom as dom
 import logging
 import codecs
 import re
-from datatypes import *
+from datatypes import NodeId, valueIsInternalType
 from nodes import *
 from opaque_type_mapping import opaque_type_mapping
 
+from type_parser import CSVBSDTypeParser
+import io
+import tempfile
+import base64
+
 __all__ = ['NodeSet', 'getSubTypesOf']
 
 logger = logging.getLogger(__name__)
@@ -36,7 +34,7 @@ if sys.version_info[0] >= 3:
         return s
     string_types = str
 else:
-    string_types = basestring 
+    string_types = basestring
 
 ####################
 # Helper Functions #
@@ -120,7 +118,7 @@ class NodeSet(object):
         self.nodes = {}
         self.aliases = {}
         self.namespaces = ["http://opcfoundation.org/UA/"]
-        self.namespaceMapping = {};
+        self.namespaceMapping = {}
 
     def sanitize(self):
         for n in self.nodes.values():
@@ -135,6 +133,7 @@ class NodeSet(object):
                 if not ref.referenceType in self.nodes:
                     raise Exception("Reference " + str(ref) + " has an unknown reference type")
                 if not ref.target in self.nodes:
+                    print(self.namespaces)
                     raise Exception("Reference " + str(ref) + " has an unknown target")
 
     def addNamespace(self, nsURL):
@@ -144,20 +143,13 @@ class NodeSet(object):
     def createNamespaceMapping(self, orig_namespaces):
         """Creates a dict that maps from the nsindex in the original nodeset to the
            nsindex in the combined nodeset"""
-        m = {}
+        self.namespaceMapping = {}
         for index, name in enumerate(orig_namespaces):
-            m[index] = self.namespaces.index(name)
-        return m
+            self.namespaceMapping[index] = self.namespaces.index(name)
 
     def getNodeByBrowseName(self, idstring):
         return next((n for n in self.nodes.values() if idstring == n.browseName.name), None)
 
-    def getNodeById(self, namespace, id):
-        nodeId = NodeId()
-        nodeId.ns = namespace
-        nodeId.i = id
-        return self.nodes[nodeId]
-
     def getRoot(self):
         return self.getNodeByBrowseName("Root")
 
@@ -208,11 +200,52 @@ class NodeSet(object):
             result.update(dictionary)
         return result
 
+    def getNodeByIDString(self, idStr):
+        # Split id to namespace part and id part
+        m = re.match("ns=([^;]+);(.*)", idStr)
+        if m:
+            ns = m.group(1)
+            # Convert namespace uri to index
+            if not ns.isdigit():
+                if ns not in self.namespaces:
+                    return None
+                ns = self.namespaces.index(ns)
+                idStr = "ns={};{}".format(ns, m.group(2))
+        nodeId = NodeId(idStr)
+        if not nodeId in self.nodes:
+            return None
+        return self.nodes[nodeId]
+
+    def remove_node(self, node):
+
+        def filterRef(r, rt):
+            return (r.referenceType != rt.referenceType) or (not (
+                    rt.target == node.id or rt.source == node.id
+                ))
+
+        for r in node.references:
+            if r.target == node.id:
+                if r.source not in self.nodes:
+                    continue
+                self.nodes[r.source].references = set(filter(
+                    lambda rt: filterRef(r, rt),
+                    self.nodes[r.source].references
+                ))
+            elif r.source == node.id:
+                if r.target not in self.nodes:
+                    continue
+                self.nodes[r.target].references = set(filter(
+                    lambda rt: filterRef(r, rt),
+                    self.nodes[r.target].references
+                ))
+        del self.nodes[node.id]
+
+
     def addNodeSet(self, xmlfile, hidden=False, typesArray="UA_TYPES"):
         # Extract NodeSet DOM
 
         fileContent = xmlfile.read()
-        # Remove BOM since the dom parser cannot handle it on python 3 windows
+        # Remove BOM since the dom parser cannot handle it on Python 3 Windows
         if fileContent.startswith( codecs.BOM_UTF8 ):
             fileContent = fileContent.lstrip( codecs.BOM_UTF8 )
         if (sys.version_info >= (3, 0)):
@@ -246,7 +279,7 @@ class NodeSet(object):
 
         for ns in orig_namespaces:
             self.addNamespace(ns)
-        self.namespaceMapping[modelUri] = self.createNamespaceMapping(orig_namespaces)
+        self.createNamespaceMapping(orig_namespaces) # mapping for this file
 
         # Extract the aliases
         for nd in nodeset.childNodes:
@@ -257,7 +290,7 @@ class NodeSet(object):
                 self.aliases = self.merge_dicts(self.aliases, buildAliasList(nd))
 
         # Instantiate nodes
-        newnodes = []
+        newnodes = {}
         for nd in nodeset.childNodes:
             if nd.nodeType != nd.ELEMENT_NODE:
                 continue
@@ -265,14 +298,23 @@ class NodeSet(object):
             if not node:
                 continue
             node.replaceAliases(self.aliases)
-            node.replaceNamespaces(self.namespaceMapping[modelUri])
+            node.replaceNamespaces(self.namespaceMapping)
             node.typesArray = typesArray
 
             # Add the node the the global dict
             if node.id in self.nodes:
                 raise Exception("XMLElement with duplicate ID " + str(node.id))
             self.nodes[node.id] = node
-            newnodes.append(node)
+            newnodes[node.id] = node
+
+        # Parse Datatypes in order to find out what the XML keyed values actually
+        # represent.
+        # Ex. <rpm>123</rpm> is not encodable
+        #     only after parsing the datatypes, it is known that
+        #     rpm is encoded as a double
+        for n in newnodes.values():
+            if isinstance(n, DataTypeNode):
+                n.buildEncoding(self, namespaceMapping=self.namespaceMapping)
 
     def getBinaryEncodingIdForNode(self, nodeId):
         """
@@ -288,18 +330,6 @@ class NodeSet(object):
                     return ref.target
         raise Exception("No DefaultBinary encoding defined for node " + str(nodeId))
 
-    def buildEncodingRules(self):
-        """ Calls buildEncoding() for all DataType nodes (opcua_node_dataType_t).
-
-            No return value
-        """
-        stat = {True: 0, False: 0}
-        for n in self.nodes.values():
-            if isinstance(n, DataTypeNode):
-                n.buildEncoding(self)
-                stat[n.isEncodable()] = stat[n.isEncodable()] + 1
-        logger.debug("Type definitions built/passed: " +  str(stat))
-
     def allocateVariables(self):
         for n in self.nodes.values():
             if isinstance(n, VariableNode):
@@ -350,7 +380,7 @@ class NodeSet(object):
     def addInverseReferences(self):
         # Ensure that every reference has an inverse reference in the target
         for u in self.nodes.values():
-            for ref in u.references:
+            for ref in u.references.copy():
                 back = Reference(ref.target, ref.referenceType, ref.source, not ref.isForward)
                 self.nodes[ref.target].references.add(back) # ref set does not make a duplicate entry
 
@@ -359,7 +389,96 @@ class NodeSet(object):
         parentreftypes = list(map(lambda x: x.id, parentreftypes))
 
         for node in self.nodes.values():
+            if node.id.ns == 0 and node.id.i in [78, 80, 84]:
+                # ModellingRule, Root node do not have a parent
+                continue
+
             parentref = node.getParentReference(parentreftypes)
             if parentref is not None:
                 node.parent = self.nodes[parentref.target]
+                if not node.parent:
+                    raise RuntimeError("Node {}: Did not find parent node: ".format(str(node.id)))
                 node.parentReference = self.nodes[parentref.referenceType]
+            # Some nodes in the full nodeset do not have a parent. So accept this and do not show an error.
+            #else:
+            #    raise RuntimeError("Node {}: HierarchicalReference (or subtype of it) to parent node is missing.".format(str(node.id)))
+
+    def generateParser(self, existing, infiles , bsdFile):
+        self.all_files = []
+        import_bsd = []
+        type_bsd = []
+        for xmlfile in existing:
+            if xmlfile.name == "deps/ua-nodeset/DI/Opc.Ua.Di.NodeSet2.xml":
+                continue
+            nodeset_base = open(xmlfile.name, "rb")
+            #fileContent = xmlfile.read()
+            fileContent = nodeset_base.read()
+            # Remove BOM since the dom parser cannot handle it on python 3 windows
+            if fileContent.startswith( codecs.BOM_UTF8 ):
+                fileContent = fileContent.lstrip( codecs.BOM_UTF8 )
+            if (sys.version_info >= (3, 0)):
+                fileContent = fileContent.decode("utf-8")
+
+            # Remove the uax namespace from tags. UaModeler adds this namespace to some elements
+            fileContent = re.sub(r"<([/]?)uax:(.+?)([/]?)>", "<\g<1>\g<2>\g<3>>", fileContent)
+
+            nodesets = dom.parseString(fileContent).getElementsByTagName("UANodeSet")
+            if len(nodesets) == 0 or len(nodesets) > 1:
+                raise Exception("contains no or more then 1 nodeset")
+            nodeset = nodesets[0]
+            variableNodes = nodeset.getElementsByTagName("UAVariable")
+            for nd in variableNodes:
+                if (nd.hasAttribute("SymbolicName") and (re.match(r".*_BinarySchema", nd.attributes["SymbolicName"].nodeValue) or nd.attributes["SymbolicName"].nodeValue == "TypeDictionary_BinarySchema")) or (nd.hasAttribute("ParentNodeId") and not nd.hasAttribute("SymbolicName") and re.fullmatch(r"i=93", nd.attributes["ParentNodeId"].nodeValue)):
+                    type_content = nd.getElementsByTagName("Value")[0].getElementsByTagName("ByteString")[0]
+                    f = tempfile.NamedTemporaryFile(delete=False, suffix='.bsd')
+                    f.write(base64.b64decode(type_content.firstChild.nodeValue))
+                    f.flush()
+                    self.all_files.append(f.name)
+                    f.close()
+                    bsd = "UA_TYPES#" + f.name
+                    import_bsd.append(bsd)
+        for xmlfile in infiles:
+            nodeset_base = open(xmlfile.name, "rb")
+            #fileContent = xmlfile.read()
+            fileContent = nodeset_base.read()
+            # Remove BOM since the dom parser cannot handle it on python 3 windows
+            if fileContent.startswith( codecs.BOM_UTF8 ):
+                fileContent = fileContent.lstrip( codecs.BOM_UTF8 )
+            if (sys.version_info >= (3, 0)):
+                fileContent = fileContent.decode("utf-8")
+
+            # Remove the uax namespace from tags. UaModeler adds this namespace to some elements
+            fileContent = re.sub(r"<([/]?)uax:(.+?)([/]?)>", "<\g<1>\g<2>\g<3>>", fileContent)
+
+            nodesets = dom.parseString(fileContent).getElementsByTagName("UANodeSet")
+            if len(nodesets) == 0 or len(nodesets) > 1:
+                raise Exception("contains no or more then 1 nodeset")
+            nodeset = nodesets[0]
+            variableNodes = nodeset.getElementsByTagName("UAVariable")
+            for nd in variableNodes:
+                if (nd.hasAttribute("SymbolicName") and (re.match(r".*_BinarySchema", nd.attributes["SymbolicName"].nodeValue) or nd.attributes["SymbolicName"].nodeValue == "TypeDictionary_BinarySchema")) or (nd.hasAttribute("ParentNodeId") and not nd.hasAttribute("SymbolicName") and re.fullmatch(r"i=93", nd.attributes["ParentNodeId"].nodeValue)):
+                    type_content = nd.getElementsByTagName("Value")[0].getElementsByTagName("ByteString")[0]
+                    f = tempfile.NamedTemporaryFile(suffix='.bsd')
+                    f.write(base64.b64decode(type_content.firstChild.nodeValue))
+                    f.flush()
+                    f.seek(0)
+                    bf = io.BufferedReader(f)
+                    self.all_files.append(f.name)
+                    tmp = io.TextIOWrapper(bf, 'UTF-8', newline='\n')
+                    tmp.mode = 'r'
+                    type_bsd.append(tmp)
+
+        opaque_map = []
+        selected_types = []
+        type_csv = []
+        no_builtin = True
+        outname = "outname"
+
+        for bsd in bsdFile:
+            type_bsd.append(bsd)
+
+        self.parser = CSVBSDTypeParser(opaque_map, selected_types, no_builtin, outname, import_bsd,
+                                    type_bsd, type_csv, self.namespaces)
+        self.parser.create_types()
+
+        nodeset_base.close()
