@@ -1,5 +1,5 @@
 /* ========================================================================
- * Copyright (c) 2005-2019 The OPC Foundation, Inc. All rights reserved.
+ * Copyright (c) 2005-2021 The OPC Foundation, Inc. All rights reserved.
  *
  * OPC Foundation MIT License 1.00
  * 
@@ -767,6 +767,11 @@ OPCUA_BEGIN_EXTERN_C
  *===========================================================================*/
 #define OpcUa_BadSequenceNumberUnknown 0x807A0000
 
+/*============================================================================
+ * The Server does not support retransmission queue and acknowledgement of sequence numbers is not available.
+ *===========================================================================*/
+#define OpcUa_GoodRetransmissionQueueNotSupported 0x00DF0000
+
 /*============================================================================
  * The requested notification message is no longer available.
  *===========================================================================*/
@@ -803,7 +808,7 @@ OPCUA_BEGIN_EXTERN_C
 #define OpcUa_BadTcpSecureChannelUnknown 0x807F0000
 
 /*============================================================================
- * The size of the message specified in the header is too large.
+ * The size of the message chunk specified in the header is too large.
  *===========================================================================*/
 #define OpcUa_BadTcpMessageTooLarge 0x80800000
 
@@ -1087,6 +1092,16 @@ OPCUA_BEGIN_EXTERN_C
  *===========================================================================*/
 #define OpcUa_BadRequestNotComplete 0x81130000
 
+/*============================================================================
+ * The device identity needs a ticket before it can be accepted.
+ *===========================================================================*/
+#define OpcUa_BadTicketRequired 0x811F0000
+
+/*============================================================================
+ * The device identity needs a ticket before it can be accepted.
+ *===========================================================================*/
+#define OpcUa_BadTicketInvalid 0x81200000
+
 /*============================================================================
  * The value does not come from the real source and has been edited by the server.
  *===========================================================================*/
@@ -1252,6 +1267,61 @@ OPCUA_BEGIN_EXTERN_C
  *===========================================================================*/
 #define OpcUa_BadMaxConnectionsReached 0x80B70000
 
+/*============================================================================
+ * The value may not be accurate because the transducer is in manual mode.
+ *===========================================================================*/
+#define OpcUa_UncertainTransducerInManual 0x42080000
+
+/*============================================================================
+ * The value is simulated.
+ *===========================================================================*/
+#define OpcUa_UncertainSimulatedValue 0x42090000
+
+/*============================================================================
+ * The value may not be accurate due to a sensor calibration fault.
+ *===========================================================================*/
+#define OpcUa_UncertainSensorCalibration 0x420A0000
+
+/*============================================================================
+ * The value may not be accurate due to a configuration issue.
+ *===========================================================================*/
+#define OpcUa_UncertainConfigurationError 0x420F0000
+
+/*============================================================================
+ * The value source supports cascade handshaking and the value has been Initialized based on an initialization request from a cascade secondary.
+ *===========================================================================*/
+#define OpcUa_GoodCascadeInitializationAcknowledged 0x04010000
+
+/*============================================================================
+ * The value source supports cascade handshaking and is requesting initialization of a cascade primary.
+ *===========================================================================*/
+#define OpcUa_GoodCascadeInitializationRequest 0x04020000
+
+/*============================================================================
+ * The value source supports cascade handshaking, however, the source’s current state does not allow for cascade.
+ *===========================================================================*/
+#define OpcUa_GoodCascadeNotInvited 0x04030000
+
+/*============================================================================
+ * The value source supports cascade handshaking, however, the source has not selected the corresponding cascade primary for use.
+ *===========================================================================*/
+#define OpcUa_GoodCascadeNotSelected 0x04040000
+
+/*============================================================================
+ * There is a fault state condition active in the value source.
+ *===========================================================================*/
+#define OpcUa_GoodFaultStateActive 0x04070000
+
+/*============================================================================
+ * A fault state condition is being requested of the destination.
+ *===========================================================================*/
+#define OpcUa_GoodInitiateFaultState 0x04080000
+
+/*============================================================================
+ * The value is accurate, and the signal source supports cascade handshaking.
+ *===========================================================================*/
+#define OpcUa_GoodCascade 0x04090000
+
 OPCUA_END_EXTERN_C
 
 #endif /* _OpcUa_StatusCodes_H_ */
